==> rccn_gen-1.0.1.tar.gz: rccn_gen-1.0.1/src/rccn_gen/utils.py <==
import re
import inflect
import os
from yamcs.pymdb import IntegerArgument, FloatArgument, BooleanArgument, EnumeratedArgument, StringArgument


def to_upper_camel_case(s):
    if s[0].islower():
        s = s[0].upper() + s[1:]
    if ' ' in s or '_' in s:
        words = re.split(r'[^a-zA-Z0-9]', s)
        return ''.join(word.capitalize() for word in words if word)
    return s

def to_snake_case(s):
    s = re.sub(r'(?<!^)(?=[A-Z])', '_', s).lower()
    s = re.sub(r'[^a-zA-Z0-9_]', '_', s)
    s = re.sub(r'__', '_', s)
    return s

def replace_with_indentation(text, keyword, replacement):
    lines = text.split('\n')
    indent = 0
    for line in lines:
        if keyword in line:
            indent = len(line) - len(line.lstrip())
    
    return text.replace(keyword, replacement.replace('\n', ('\n'+(indent*' '))))

def insert_before_with_indentation(text, keyword, replacement):
    lines = text.split('\n')
    indent = 0
    for line in lines:
        if keyword in line:
            indent = len(line) - len(line.lstrip())
    return text.replace(keyword, (replacement.replace('\n', ('\n'+(indent*' ')))+keyword))

def get_keywords(text):
    pattern = r'<<.*?>>'
    return re.findall(pattern, text)

def get_var_keywords(text):
    pattern = r'<<VAR_.*?>>'
    return re.findall(pattern, text)

def get_service_module_keywords(text):
    pattern = r'<<SERVICE_MODULE_.*?>>'
    return re.findall(pattern, text)

def get_command_module_keywords(text):
    pattern = r'<<COMMAND_MODULE_.*?>>'
    return re.findall(pattern, text)

def delete_all_keywords(text):
        keywords = get_keywords(text)
        for keyword in keywords:
            text = text.replace(keyword, '')
        return text

def delete_all_command_module_keywords(text):
        keywords = get_command_module_keywords(text)
        for keyword in keywords:
            text = text.replace(keyword, '')
        return text

def arg_type_to_rust(arg, bit_number_str='32'):
    if isinstance(arg, IntegerArgument):
        if arg.signed:
            return 'i'+bit_number_str
        else:
            return 'u'+bit_number_str
    elif isinstance(arg, FloatArgument):
        return 'f'+bit_number_str
    elif isinstance(arg, BooleanArgument):
        return 'bool'
    elif isinstance(arg, EnumeratedArgument):
        return arg.name
    elif isinstance(arg, StringArgument):
        return 'String'
    else:
        print('Argument type is not supported: '+str(type(arg)))
        return None
    
def arg_enum_rust_definition(arg):
    if not isinstance(arg, EnumeratedArgument):
        raise ValueError('Provided Argument is not of type EnumeratedArgument.')
    definition_text = 'pub enum '+arg.name+' {\n'
    for choice in arg.choices:
        definition_text += ('\t'+str(choice[1])+' = '+str(choice[0])+',\n')
    definition_text += '}\n'
    return definition_text

def engineering_bit_number(raw_bit_number):
    if raw_bit_number < 1 or raw_bit_number > 128:
        raise ValueError("raw_bit_number must be between 1 and 128")
    power = 1
    while 2**power < raw_bit_number:
        power += 1
    bit_number = 2**power
    return (bit_number)

def get_data_type(parent_classes):
    """Extract the data type from a list of parent class names.
    Returns the class name that ends with 'DataType' or None if not found."""
    for class_name in parent_classes:
        if class_name.endswith('DataType'):
            return class_name
    return None

def get_base_type(parent_classes):
    for base_type in ["Argument", "Member", "Parameter"]:
        if base_type in parent_classes:
            return base_type
    if "DataType" in parent_classes:
        return "DataType"
    return None

def rust_type_definition(pymdb_data_instance, parent_name="MyStruct"):
    parent_classes = list(map(lambda type: type.__name__, type(pymdb_data_instance).mro()))
    data_type = get_data_type(parent_classes)
    base_type = get_base_type(parent_classes)
    if base_type is None:
        base_type = pymdb_data_instance.__class__.__name__
    if data_type is None:
        raise ValueError("RCCN-Gen: Data type not found in parent classes.")
    if not hasattr(pymdb_data_instance, 'name') or pymdb_data_instance.name is None:
        p = inflect.engine()
        singular_name = p.singular_noun(parent_name)
        if singular_name is False:
            pymdb_data_instance.name = parent_name
        else:
            pymdb_data_instance.name = singular_name
        print("RCCN-Gen: Information: An unnamed "+base_type+" has been named \""+to_upper_camel_case(pymdb_data_instance.name)+"\" in the generated RCCN code.")
    sc_instance_name = to_snake_case(pymdb_data_instance.name)

    if data_type == 'IntegerDataType':
        if pymdb_data_instance.encoding is None or pymdb_data_instance.encoding.bits is None:
            raw_bit_number = 8
            print("RCCN-Gen: Warning: No encoding for "+base_type+" "+pymdb_data_instance.name+" found. Using 8 as default for raw bit number.")
        else:
            raw_bit_number = pymdb_data_instance.encoding.bits
        raw_bit_number_str = str(raw_bit_number)
        eng_bit_number = engineering_bit_number(raw_bit_number)
        eng_bit_number_str = str(eng_bit_number)
        definition_text = ["\t#[bits("+raw_bit_number_str+")]\n"]
        if pymdb_data_instance.signed:
            definition_text[0] += ("\tpub "+sc_instance_name+": i"+eng_bit_number_str+",\n")
        else:
            definition_text[0] += ("\tpub "+sc_instance_name+": u"+eng_bit_number_str+",\n")
        definition_text.append("")
    
    elif data_type == 'BooleanDataType':
        definition_text = ["\t#[bits(1)]\n\tpub "+sc_instance_name+": bool,\n"]
        definition_text.append("")
    
    elif data_type == 'StringDataType':
        definition_text = ["\t#[null_terminated]\n\tpub "+sc_instance_name+": String,\n", ""]
    
    elif data_type == 'ArrayDataType':
        definition_text = rust_type_definition(pymdb_data_instance.data_type, parent_name=pymdb_data_instance.name)
        definition_text[0] = definition_text[0].replace(': ', ': Vec<').replace(',\n', '>,\n')
    
    elif data_type == 'EnumeratedDataType':
        definition_text = ["\tpub "+pymdb_data_instance.name+": "+to_upper_camel_case(pymdb_data_instance.name)+",\n"]
        definition_text.append("pub enum "+to_upper_camel_case(pymdb_data_instance.name)+" {\n")
        for choice in pymdb_data_instance.choices:
            definition_text[1] += "\t"+str(choice[1])+" = "+str(choice[0])+",\n"
        definition_text[1] += "}\n"
    
    elif data_type == 'AggregateDataType':
        struct_name = to_upper_camel_case(pymdb_data_instance.name)
        definition_text = ["\tpub "+sc_instance_name+": "+struct_name+",\n"]
        definition_text.append("pub struct "+struct_name+" {\n")
        for member in pymdb_data_instance.members:
            definition_text[1] += rust_type_definition(member, parent_name=pymdb_data_instance.name)[0]
        definition_text[1] += "}\n"
        for member in pymdb_data_instance.members:
            definition_text[1] += rust_type_definition(member, parent_name=pymdb_data_instance.name)[1]

    else:
        definition_text = ["\t// Please implement datatype "+data_type+" here.\n", ""]
    return definition_text

==> rccn_gen-1.1.0.tar.gz: rccn_gen-1.1.0/src/rccn_gen/utils.py <==
import re
import inflect
import os
from caseconverter import *
from yamcs.pymdb import IntegerArgument, FloatArgument, BooleanArgument, EnumeratedArgument, StringArgument


def to_upper_camel_case(s):
    if s[0].islower():
        s = s[0].upper() + s[1:]
    if ' ' in s or '_' in s:
        words = re.split(r'[^a-zA-Z0-9]', s)
        return ''.join(word.capitalize() for word in words if word)
    return s

def to_snake_case(s):
    s = re.sub(r'(?<!^)(?=[A-Z])', '_', s).lower()
    s = re.sub(r'[^a-zA-Z0-9_]', '_', s)
    s = re.sub(r'__', '_', s)
    return s

def replace_with_indentation(text, keyword, replacement):
    lines = text.split('\n')
    indent = 0
    for line in lines:
        if keyword in line:
            indent = len(line) - len(line.lstrip())
    
    return text.replace(keyword, replacement.replace('\n', ('\n'+(indent*' '))))

def insert_before_with_indentation(text, keyword, replacement):
    lines = text.split('\n')
    indent = 0
    for line in lines:
        if keyword in line:
            indent = len(line) - len(line.lstrip())
    return text.replace(keyword, (replacement.replace('\n', ('\n'+(indent*' ')))+keyword))

def get_keywords(text):
    pattern = r'<<.*?>>'
    return re.findall(pattern, text)

def get_var_keywords(text):
    pattern = r'<<VAR_.*?>>'
    return re.findall(pattern, text)

def get_service_module_keywords(text):
    pattern = r'<<SERVICE_MODULE_.*?>>'
    return re.findall(pattern, text)

def get_command_module_keywords(text):
    pattern = r'<<COMMAND_MODULE_.*?>>'
    return re.findall(pattern, text)

def delete_all_keywords(text):
        keywords = get_keywords(text)
        for keyword in keywords:
            text = text.replace(keyword, '')
        return text

def delete_all_command_module_keywords(text):
        keywords = get_command_module_keywords(text)
        for keyword in keywords:
            text = text.replace(keyword, '')
        return text

def arg_type_to_rust(arg, bit_number_str='32'):
    if isinstance(arg, IntegerArgument):
        if arg.signed:
            return 'i'+bit_number_str
        else:
            return 'u'+bit_number_str
    elif isinstance(arg, FloatArgument):
        return 'f'+bit_number_str
    elif isinstance(arg, BooleanArgument):
        return 'bool'
    elif isinstance(arg, EnumeratedArgument):
        return arg.name
    elif isinstance(arg, StringArgument):
        return 'String'
    else:
        print('Argument type is not supported: '+str(type(arg)))
        return None
    
def arg_enum_rust_definition(arg):
    if not isinstance(arg, EnumeratedArgument):
        raise ValueError('Provided Argument is not of type EnumeratedArgument.')
    definition_text = 'pub enum '+arg.name+' {\n'
    for choice in arg.choices:
        definition_text += ('\t'+str(choice[1])+' = '+str(choice[0])+',\n')
    definition_text += '}\n'
    return definition_text

def engineering_bit_number(raw_bit_number):
    if raw_bit_number < 1 or raw_bit_number > 128:
        raise ValueError("raw_bit_number must be between 1 and 128")
    power = 1
    while 2**power < raw_bit_number:
        power += 1
    bit_number = 2**power
    return (bit_number)

def get_data_type(parent_classes):
    """Extract the data type from a list of parent class names.
    Returns the class name that ends with 'DataType' or None if not found."""
    for class_name in parent_classes:
        if class_name.endswith('DataType'):
            return class_name
    return None

def get_base_type(parent_classes):
    for base_type in ["Argument", "Member", "Parameter"]:
        if base_type in parent_classes:
            return base_type
    if "DataType" in parent_classes:
        return "DataType"
    return None

def rust_type_definition(pymdb_data_instance, parent_name="MyStruct"):
    parent_classes = list(map(lambda type: type.__name__, type(pymdb_data_instance).mro()))
    data_type = get_data_type(parent_classes)
    base_type = get_base_type(parent_classes)
    if base_type is None:
        base_type = pymdb_data_instance.__class__.__name__
    if data_type is None:
        raise ValueError("RCCN-Gen: Data type not found in parent classes.")
    if not hasattr(pymdb_data_instance, 'name') or pymdb_data_instance.name is None:
        p = inflect.engine()
        singular_name = p.singular_noun(parent_name)
        if singular_name is False:
            pymdb_data_instance.name = parent_name
        else:
            pymdb_data_instance.name = singular_name
        print("RCCN-Gen: Information: An unnamed "+base_type+" has been named \""+pascalcase(pymdb_data_instance.name)+"\" in the generated RCCN code.")
    sc_instance_name = snakecase(pymdb_data_instance.name)
    definition_text = ["",""]
    if pymdb_data_instance.short_description is not None:
        definition_text[0] += ("\n\t/// "+str(pymdb_data_instance.short_description)+"\n")
    if data_type == 'IntegerDataType':
        if pymdb_data_instance.encoding is None or pymdb_data_instance.encoding.bits is None:
            raw_bit_number = 8
            print("RCCN-Gen: Warning: No encoding for "+base_type+" "+pymdb_data_instance.name+" found. Using 8 as default for raw bit number.")
        else:
            raw_bit_number = pymdb_data_instance.encoding.bits
        raw_bit_number_str = str(raw_bit_number)
        eng_bit_number = engineering_bit_number(raw_bit_number)
        eng_bit_number_str = str(eng_bit_number)
        definition_text[0] += "\t#[bits("+raw_bit_number_str+")]\n"
        if pymdb_data_instance.signed:
            definition_text[0] += ("\tpub "+sc_instance_name+": i"+eng_bit_number_str+",\n")
        else:
            definition_text[0] += ("\tpub "+sc_instance_name+": u"+eng_bit_number_str+",\n")
    
    elif data_type == 'BooleanDataType':
        definition_text[0] += ("\t#[bits(1)]\n\tpub "+sc_instance_name+": bool,\n")
    
    elif data_type == 'StringDataType':
        definition_text[0] += "\t#[null_terminated]\n\tpub "+sc_instance_name+": String,\n"
    
    elif data_type == 'ArrayDataType':
        definition_text = rust_type_definition(pymdb_data_instance.data_type, parent_name=pymdb_data_instance.name)
        definition_text[0] = definition_text[0].replace(': ', ': Vec<').replace(',\n', '>,\n')
        if pymdb_data_instance.short_description is not None:
            definition_text[0] = "\n\t/// "+pymdb_data_instance.short_description+"\n"+definition_text[0]
        if pymdb_data_instance.long_description is not None:
            definition_text[1] = "/// "+pymdb_data_instance.long_description+"\n" + definition_text[1]
    
    elif data_type == 'EnumeratedDataType':
        definition_text[0] += "\t#[bits("+str(pymdb_data_instance.encoding.bits)+")]\n"
        definition_text[0] += "\tpub "+pymdb_data_instance.name+": "+pascalcase(pymdb_data_instance.name)+",\n"
        definition_text[1] += ("#[derive(FromPrimative, ToPrimative, Debug)]\npub enum "+pascalcase(pymdb_data_instance.name)+" {\n")
        for choice in pymdb_data_instance.choices:
            definition_text[1] += "\t"+str(choice[1])+" = "+str(choice[0])+",\n"
        definition_text[1] += "}\n\n"
        if pymdb_data_instance.long_description is not None:
            definition_text[1] = "/// "+pymdb_data_instance.long_description+"\n" + definition_text[1]
    
    elif data_type == 'AggregateDataType':
        struct_name = pascalcase(pymdb_data_instance.name)
        definition_text[0] += "\tpub "+sc_instance_name+": "+struct_name+",\n"
        if pymdb_data_instance.long_description is not None:
            definition_text[1] += "\t/// "+pymdb_data_instance.long_description+"\n"
        definition_text[1] += ("#[derive(FromPrimative, ToPrimative, Debug)]\n")
        definition_text[1] += ("pub struct "+struct_name+" {\n")
        insert, append = ["",""]
        for member in pymdb_data_instance.members:
            mem_def = rust_type_definition(member, parent_name=pymdb_data_instance.name)
            insert += mem_def[0]
            append += mem_def[1]
        definition_text[1] += insert
        definition_text[1] += "}\n\n"
        definition_text[1] += append
    else:
        definition_text = ["\t// Please implement datatype "+data_type+" here.\n", ""]
    return definition_text
